import abc

import six

from django.utils.translation import ugettext as _

from .. import widgets


class ImageBackend(six.with_metaclass(abc.ABCMeta)):
    """
    Abstract class to expose the expected methods and properties that a custom
    backend should provide.
    """

    exceptions_to_catch = (IOError, )

    WIDGETS = {
        'foreign_key': widgets.CropForeignKeyWidget,
        'hidden': widgets.HiddenImageCropWidget,
        'ImageField': widgets.ImageCropWidget,
        'ImageCropField': widgets.ImageCropWidget,
    }

    def __init__(self, **kwargs):
        self.kwargs = kwargs
        for k, v in kwargs.items():
            setattr(self, k, v)

    @abc.abstractmethod
    def get_thumbnail_url(self, image_path, thumbnail_options):
        pass

    @abc.abstractmethod
    def get_size(self, image):
        pass

    def get_widget(self, db_field, target, admin_site):
        "Get the widget from a db_field"
        if target['fk_field']:
            # it's a ForeignKey
            return self.WIDGETS['foreign_key'](
                db_field.rel,
                field_name=target['fk_field'],
                admin_site=admin_site,
            )
        elif target['hidden']:
            # it's a hidden ImageField
            return self.WIDGETS['hidden']
        else:
            # it's a regular image field, get from its class name
            class_name = type(db_field).__name__
            if class_name not in self.WIDGETS:
                msg = _("There's no widget registered to the class {class_name}").format(
                    class_name=class_name)
                raise ValueError(msg)
            return self.WIDGETS[class_name]



from django.core.exceptions import ImproperlyConfigured
from django.utils.module_loading import import_string
from django.utils.translation import ugettext as _

from .config import settings


def max_cropping(width, height, image_width, image_height, free_crop=False):
    if free_crop:
        return [0, 0, image_width, image_height]

    ratio = width / float(height)
    if image_width < image_height * ratio:
        # width fits fully, height needs to be cropped
        offset = int(round((image_height - (image_width / ratio)) / 2))
        return [0, offset, image_width, image_height - offset]

    # height fits fully, width needs to be cropped
    offset = int(round((image_width - (image_height * ratio)) / 2))
    return [offset, 0, image_width - offset, image_height]


def get_backend():
    try:
        cls = import_string(settings.IMAGE_CROPPING_BACKEND)
    except ImportError as e:
        raise ImproperlyConfigured(
            _("Can't retrieve the image backend '{}'. Message: '{}'.").format(
                settings.IMAGE_CROPPING_BACKEND, e))

    return cls(**settings.IMAGE_CROPPING_BACKEND_PARAMS)



from __future__ import unicode_literals

from .utils import get_backend


class ImageCroppingMixin(object):
    def formfield_for_dbfield(self, db_field, **kwargs):
        crop_fields = getattr(self.model, 'crop_fields', {})
        if db_field.name in crop_fields:
            target = crop_fields[db_field.name]
            kwargs['widget'] = get_backend().get_widget(db_field, target, self.admin_site)

        return super(ImageCroppingMixin, self).formfield_for_dbfield(db_field, **kwargs)



"""
Backend for easy_thumbnails_ package. This module can't be named
"easy_thumbnails" in order to avoid Python import conflicts.

.. _easy_thumbnails: https://github.com/SmileyChris/easy-thumbnails
"""

from easy_thumbnails.exceptions import InvalidImageFormatError
from easy_thumbnails.files import get_thumbnailer
from easy_thumbnails.source_generators import pil_image

from .base import ImageBackend


class EasyThumbnailsBackend(ImageBackend):
    exceptions_to_catch = (InvalidImageFormatError, IOError)

    def get_thumbnail_url(self, image_path, thumbnail_options):
        thumb = get_thumbnailer(image_path)
        return thumb.get_thumbnail(thumbnail_options).url

    def get_size(self, image):
        return pil_image(image).size
